@@ -1,74 +1,42 @@
-{-| For further information see the <http://www.cse.chalmers.se/~rjmh/QuickCheck/manual.html QuickCheck manual>.
+{-|
+The <http://www.cse.chalmers.se/~rjmh/QuickCheck/manual.html QuickCheck manual>
+gives detailed information about using QuickCheck effectively.
 
-To use QuickCheck to check a property, first define a function
-expressing that property (functions expressing properties under test
-tend to be prefixed with @prop_@). Testing that @n + m = m + n@ holds
-for @Integer@s one might write:
+To start using QuickCheck, write down your property as a function returning @Bool@.
+For example, to check that reversing a list twice gives back the same list you can write:
 
 @
 import Test.QuickCheck
 
-prop_commutativeAdd :: Integer -> Integer -> Bool
-prop_commutativeAdd n m = n + m == m + n
+prop_reverse :: [Int] -> Bool
+prop_reverse xs = reverse (reverse xs) == xs
 @
 
-and testing:
+You can then use QuickCheck to test @prop_reverse@ on 100 random lists:
 
->>> quickCheck prop_commutativeAdd
+>>> quickCheck prop_reverse
 +++ OK, passed 100 tests.
 
-which tests @prop_commutativeAdd@ on 100 random @(Integer, Integer)@ pairs.
+To run more tests you can use the 'withMaxSuccess' combinator:
 
-'verboseCheck' can be used to see the actual values generated:
+>>> quickCheck (withMaxSuccess 10000 prop_reverse)
++++ OK, passed 10000 tests.
 
->>> verboseCheck prop_commutativeAdd
-Passed:
-0
-0
-  …98 tests omitted…
-Passed:
--68
-6
-+++ OK, passed 100 tests.
-
-and if more than 100 tests are needed the number of tests can be
-increased by updating the 'stdArgs' record:
-
->>> quickCheckWith stdArgs { maxSuccess = 500 } prop_commutativeAdd
-+++ OK, passed 500 tests.
-
-To let QuickCheck generate values of your own data type an 'Arbitrary'
-instance must be defined:
-
-@
-data Point = MkPoint Int Int deriving Eq
-
-instance Arbitrary Point where
-  arbitrary = do
-    x <- 'arbitrary'
-    y <- arbitrary
-    return (MkPoint x y)
-
-swapPoint :: Point -> Point
-swapPoint (MkPoint x y) = MkPoint y x
-
--- swapPoint . swapPoint = id
-prop_swapInvolution point = swapPoint (swapPoint point) == point
-@
-
->>> quickCheck prop_swapInvolution
-+++ OK, passed 100 tests.
-
-See "Test.QuickCheck.Function" for generating random shrinkable,
-showable functions used for testing higher-order functions and
-"Test.QuickCheck.Monadic" for testing impure or monadic code
-(e.g. effectful code in 'IO').
+To use QuickCheck on your own data types you will need to write 'Arbitrary'
+instances for those types. See the
+<http://www.cse.chalmers.se/~rjmh/QuickCheck/manual.html QuickCheck manual> for
+details about how to do that.
 
+This module exports most of QuickCheck's functionality, but see also
+"Test.QuickCheck.Monadic" which helps with testing impure or monadic code.
 -}
 {-# LANGUAGE CPP #-}
 #ifndef NO_SAFE_HASKELL
 {-# LANGUAGE Safe #-}
 #endif
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
+{-# LANGUAGE PatternSynonyms #-}
+#endif
 module Test.QuickCheck
   (
     -- * Running tests
@@ -103,9 +71,11 @@ module Test.QuickCheck
   , elements
   , growingElements
   , sized
+  , getSize
   , resize
   , scale
   , suchThat
+  , suchThatMap
   , suchThatMaybe
   , listOf
   , listOf1
@@ -127,6 +97,14 @@ module Test.QuickCheck
   , Arbitrary(..)
   , CoArbitrary(..)
 
+  -- ** Unary and Binary classes
+  , Arbitrary1(..)
+  , arbitrary1
+  , shrink1
+  , Arbitrary2(..)
+  , arbitrary2
+  , shrink2
+
     -- ** Helper functions for implementing arbitrary
   , arbitrarySizedIntegral
   , arbitrarySizedNatural
@@ -135,6 +113,9 @@ module Test.QuickCheck
   , arbitraryBoundedIntegral
   , arbitraryBoundedRandom
   , arbitraryBoundedEnum
+  , arbitraryUnicodeChar
+  , arbitraryASCIIChar
+  , arbitraryPrintableChar
     -- ** Helper functions for implementing shrink
 #ifndef NO_GENERICS
   , genericCoarbitrary
@@ -144,6 +125,8 @@ module Test.QuickCheck
 #endif
   , shrinkNothing
   , shrinkList
+  , shrinkMap
+  , shrinkMapBy
   , shrinkIntegral
   , shrinkRealFrac
     -- ** Helper functions for implementing coarbitrary
@@ -170,6 +153,22 @@ module Test.QuickCheck
   , Shrinking(..)
   , ShrinkState(..)
 #endif
+  , ASCIIString(..)
+  , UnicodeString(..)
+  , PrintableString(..)
+
+    -- ** Functions
+  , Fun
+  , applyFun
+  , applyFun2
+  , applyFun3
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
+  , pattern Fn
+  , pattern Fn2
+  , pattern Fn3
+#endif
+  , Function (..)
+  , functionMap
 
     -- * Properties
   , Property, Testable(..)
@@ -179,11 +178,15 @@ module Test.QuickCheck
   , shrinking
   , (==>)
   , (===)
+#ifndef NO_DEEPSEQ
+  , total
+#endif
   , ioProperty
     -- *** Controlling property execution
   , verbose
   , once
   , again
+  , withMaxSuccess
   , within
   , noShrinking
     -- *** Conjunction and disjunction
@@ -218,8 +221,8 @@ import Test.QuickCheck.Arbitrary
 import Test.QuickCheck.Modifiers
 import Test.QuickCheck.Property hiding ( Result(..) )
 import Test.QuickCheck.Test
-import Test.QuickCheck.Text
 import Test.QuickCheck.Exception
+import Test.QuickCheck.Function
 #ifndef NO_TEMPLATE_HASKELL
 import Test.QuickCheck.All
 #endif
